@@ -4,13 +4,13 @@
  *
  *      Description:  The interface to dynamically load ptt files
  *
- *      Version:   5.0
- *      Released:  10.11.2019
+ *      Version:   5.1.0
+ *      Released:  20.11.2020
  *
  *      Author:  Thomas Gruber (tg), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -84,5 +84,6 @@ struct bstrList* dynbench_getall();
 int dynbench_test(bstring testname);
 int dynbench_load(bstring testname, TestCase **testcase, char* tmpfolder, char *compilers, char* compileflags);
 int dynbench_close(TestCase* testcase, char* tmpfolder);
+int dynbench_asm(bstring testname, char* tmpfolder, bstring outfile);
 
 #endif
