@@ -4,13 +4,13 @@
  *
  *      Description:  The interface to dynamically load ptt files
  *
- *      Version:   5.0
- *      Released:  10.11.2019
+ *      Version:   5.1.0
+ *      Released:  20.11.2020
  *
  *      Author:   Thomas Gruber (tg), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -335,7 +335,8 @@ static struct bstrList* parse_asm(TestCase* testcase, struct bstrList* input)
         struct bstrList* pre = bstrListCreate();
         struct bstrList* loop = bstrListCreate();
         int got_loop = 0;
-        bstring bLOOP = bformat("LOOP");
+        bstring bloopname = bformat("%s_loop", testcase->name);
+	bstring bLOOP = bfromcstr("LOOP");
         int step = testcase->stride;
 
         for (int i = 0; i < input->qty; i++)
@@ -354,7 +355,6 @@ static struct bstrList* parse_asm(TestCase* testcase, struct bstrList* input)
                 bstrListAdd(loop, input->entry[i]);
             }
         }
-        bdestroy(bLOOP);
 
         output = bstrListCreate();
 
@@ -364,17 +364,19 @@ static struct bstrList* parse_asm(TestCase* testcase, struct bstrList* input)
         {
             bstrListAdd(output, pre->entry[i]);
         }
-        loopheader(output, "1", step);
+        loopheader(output, bdata(bloopname), step);
         for (int i = 0; i < loop->qty; i++)
         {
             bstrListAdd(output, loop->entry[i]);
         }
-        loopfooter(output, "1", step);
+        loopfooter(output, bdata(bloopname), step);
 
         footer(output, testcase->name);
 
         bstrListDestroy(pre);
         bstrListDestroy(loop);
+        bdestroy(bLOOP);
+        bdestroy(bloopname);
     }
     return output;
 }
@@ -517,6 +519,7 @@ static int compile_file(bstring compiler, bstring flags, bstring asmfile, bstrin
     if (blength(compiler) == 0 || blength(asmfile) == 0)
         return -1;
     char buf[1024];
+    int exitcode = 0;
     bstring bstdout = bfromcstr("");
 
 
@@ -543,12 +546,16 @@ static int compile_file(bstring compiler, bstring flags, bstring asmfile, bstrin
         {
             fprintf(stderr, "%s\n", bdata(bstdout));
         }
-        pclose(fp);
+        exitcode = pclose(fp);
+    }
+    else
+    {
+        exitcode = errno;
     }
     bdestroy(cmd);
     bdestroy(bstdout);
 
-    return 0;
+    return exitcode;
 }
 
 
@@ -628,7 +635,7 @@ int dynbench_load(bstring testname, TestCase **testcase, char* tmpfolder, char *
                     if (mkdir(bdata(buildfolder), 0700) == 0)
                     {
                         int asm_written = 0;
-                        bstring asmfile = bformat("%s/%ld/%s.S", tmpfolder , pid, bdata(testname));
+                        bstring asmfile = bformat("%s/%s.S", bdata(buildfolder), bdata(testname));
 
                         struct bstrList* asmb = parse_asm(test, code);
                         if (asmb)
@@ -663,7 +670,7 @@ int dynbench_load(bstring testname, TestCase **testcase, char* tmpfolder, char *
                             {
                                 cflags = bfromcstr("");
                             }
-                            bstring objfile = bformat("%s/%ld/%s.o", tmpfolder , pid, bdata(testname));
+                            bstring objfile = bformat("%s/%s.o", bdata(buildfolder), bdata(testname));
                             cret = compile_file(compiler, cflags, asmfile, objfile);
                             if (cret == 0)
                             {
@@ -676,11 +683,13 @@ int dynbench_load(bstring testname, TestCase **testcase, char* tmpfolder, char *
                                 else
                                 {
                                     fprintf(stderr, "Cannot load function %s from %s\n", bdata(testname), bdata(objfile));
+                                    err = cret;
                                 }
                             }
                             else
                             {
                                 fprintf(stderr, "Cannot compile file %s to %s\n", bdata(asmfile), bdata(objfile));
+                                err = cret;
                             }
                             bdestroy(cflags);
                             bdestroy(objfile);
@@ -688,6 +697,7 @@ int dynbench_load(bstring testname, TestCase **testcase, char* tmpfolder, char *
                         else
                         {
                             fprintf(stderr, "Cannot find any compiler %s\n", bdata(buildfolder));
+                            err = -1;
                         }
                         bdestroy(candidates);
                         bdestroy(compiler);
@@ -711,6 +721,7 @@ int dynbench_load(bstring testname, TestCase **testcase, char* tmpfolder, char *
             else
             {
                 fprintf(stderr, "Failed to allocate space for the testname\n");
+                err = -ENOMEM;
             }
             bstrListDestroy(code);
 
@@ -718,12 +729,14 @@ int dynbench_load(bstring testname, TestCase **testcase, char* tmpfolder, char *
         else
         {
             fprintf(stderr, "Cannot read ptt file %s\n", bdata(pttfile));
+            err = -EPERM;
         }
 
     }
     else
     {
         fprintf(stderr, "Cannot open ptt file %s\n", bdata(pttfile));
+        err = -ENOENT;
     }
     bdestroy(pttfile);
 
@@ -765,3 +778,32 @@ int dynbench_close(TestCase* testcase, char* tmpfolder)
     }
     return 0;
 }
+
+int dynbench_asm(bstring testname, char* tmpfolder, bstring outfile)
+{
+    if (blength(testname) > 0 && tmpfolder && blength(outfile) > 0)
+    {
+        int ret = 0;
+        bstring asmfile = bformat("%s/%ld/%s.S", tmpfolder, getpid(), bdata(testname));
+        char buf[1024];
+        FILE* infp = fopen(bdata(asmfile), "r");
+        if (infp)
+        {
+            FILE* outfp = fopen(bdata(outfile), "w");
+            if (outfp)
+            {
+                for (;;)
+                {
+                    ret = fread(buf, sizeof(char), sizeof(buf), infp);
+                    if (ret <= 0)
+                    {
+                        break;
+                    }
+                    ret = fwrite(buf, sizeof(char), ret, outfp);
+                }
+                fclose(outfp);
+            }
+            fclose(infp);
+        }
+    }
+}
