@@ -5,15 +5,15 @@
  *
  *      Description:  Implementation of access daemon.
  *
- *      Version:   5.0
- *      Released:  10.11.2019
+ *      Version:   5.1.0
+ *      Released:  20.11.2020
  *
  *      Authors:  Michael Meier, michael.meier@rrze.fau.de
  *                Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -87,6 +87,8 @@
 #define PCM_CLIENT_IMC_DRAM_IO_REQUESTS  (0x5048)
 #define PCM_CLIENT_IMC_DRAM_DATA_READS  (0x5050)
 #define PCM_CLIENT_IMC_DRAM_DATA_WRITES (0x5054)
+#define PCM_CLIENT_IMC_PP0_TEMP (0x597C)
+#define PCM_CLIENT_IMC_PP1_TEMP (0x5980)
 #define PCM_CLIENT_IMC_MMAP_SIZE (0x6000)
 
 /* Lock file controlled from outside which prevents likwid to start.
@@ -146,6 +148,7 @@ static int getNumberOfCPUs()
         return 0;
     }
     char* ptr = fgets(buff, 1024, fpipe);
+    pclose(fpipe);
     if (ptr)
     {
         return atoi(buff);
@@ -163,6 +166,7 @@ static int getNumberOfSockets()
         return 0;
     }
     char* ptr = fgets(buff, 1024, fpipe);
+    pclose(fpipe);
     if (ptr)
     {
         return atoi(buff);
@@ -178,7 +182,7 @@ static int getNumberOfNumaNodes()
     dp = opendir ("/sys/devices/system/node/");
     if (dp != NULL)
     {
-        while ((ep = readdir (dp)))
+        while ((ep = readdir (dp)) != NULL)
         {
             if (fnmatch("node?*", ep->d_name, FNM_PATHNAME) == 0)
             {
@@ -190,6 +194,23 @@ static int getNumberOfNumaNodes()
     return count;
 }
 
+static void
+clientmem_finalize()
+{
+    if (isClientMem)
+    {
+        if (clientmem_handle >= 0)
+        {
+            if (clientmem_addr)
+            {
+                munmap(clientmem_addr, PCM_CLIENT_IMC_MMAP_SIZE);
+            }
+            close(clientmem_handle);
+            clientmem_handle = -1;
+        }
+    }
+}
+
 void __attribute__((constructor (101))) init_accessdaemon(void)
 {
 
@@ -259,6 +280,14 @@ void __attribute__((destructor (101))) close_accessdaemon(void)
         {
             if (FD_PCI[i])
             {
+                for (int j = 0; j < MAX_NUM_PCI_DEVICES; j++)
+                {
+                    if (FD_PCI[i][j] > 0)
+                    {
+                        close(FD_PCI[i][j]);
+                        FD_PCI[i][j] = 0;
+                    }
+                }
                 free(FD_PCI[i]);
                 FD_PCI[i] = NULL;
             }
@@ -268,9 +297,21 @@ void __attribute__((destructor (101))) close_accessdaemon(void)
     }
     if (FD_MSR)
     {
+        for (int i=0; i < avail_cpus; i++)
+        {
+            if (FD_MSR[i] >= 0)
+            {
+                close(FD_MSR[i]);
+                FD_MSR[i] = -1;
+            }
+        }
         free(FD_MSR);
         FD_MSR = NULL;
     }
+    if (isClientMem && clientmem_handle >= 0 && clientmem_addr)
+    {
+        clientmem_finalize();
+    }
 }
 
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
@@ -316,6 +357,7 @@ allowed_sandybridge(uint32_t reg)
     if ((allowed_intel(reg)) ||
         (((reg & 0xF00U) == 0x600U)) ||
         (((reg & 0xF00U) == 0x700U)) ||
+        (reg == MSR_PERF_STATUS)  ||
         (reg == MSR_ALT_PEBS))
     {
         return 1;
@@ -846,6 +888,25 @@ static int allowed_pci_skx(PciDeviceType type, uint32_t reg)
     return 0;
 }
 
+static int allowed_icl(uint32_t reg)
+{
+    if (allowed_sandybridge(reg))
+        return 1;
+    else
+    {
+        if (((reg & 0xF00U) == 0x700U) ||
+            ((reg & 0xF00U) == 0xA00U) ||
+            ((reg & 0xF00U) == 0xB00U) ||
+            ((reg & 0xF00U) == 0xE00U) ||
+            ((reg & 0xF00U) == 0xF00U) ||
+            (reg == MSR_PREFETCH_ENABLE) ||
+            (reg == TSX_FORCE_ABORT) ||
+            (reg == MSR_PERF_METRICS))
+            return 1;
+    }
+    return 0;
+}
+
 static int allowed_amd(uint32_t reg)
 {
     if ( (reg & 0xFFFFFFF0U) == 0xC0010000U)
@@ -907,6 +968,23 @@ allowed_amd17(uint32_t reg)
     }
 }
 
+static int
+allowed_amd17_zen2(uint32_t reg)
+{
+    if (allowed_amd17(reg))
+    {
+        return 1;
+    }
+    else if ((reg == 0xC0010208) ||
+             (reg == 0xC0010209) ||
+             (reg == 0xC001020A) ||
+             (reg == 0xC001020B))
+    {
+        return 1;
+    }
+    return 0;
+}
+
 static int
 clientmem_getStartAddr(uint64_t* startAddr)
 {
@@ -968,21 +1046,7 @@ clientmem_init()
     return 0;
 }
 
-static void
-clientmem_finalize()
-{
-    if (isClientMem)
-    {
-        if (clientmem_handle >= 0)
-        {
-            if (clientmem_addr)
-            {
-                munmap(clientmem_addr, PCM_CLIENT_IMC_MMAP_SIZE);
-            }
-            close(clientmem_handle);
-        }
-    }
-}
+
 
 static void
 clientmem_read(AccessDataRecord *dRecord)
@@ -1025,6 +1089,12 @@ clientmem_read(AccessDataRecord *dRecord)
         case 0x02:
             data = (uint64_t)*((uint32_t *)(clientmem_addr + PCM_CLIENT_IMC_DRAM_DATA_WRITES));
             break;
+        case 0x03:
+            data = (uint64_t)*((uint32_t *)(clientmem_addr + PCM_CLIENT_IMC_PP0_TEMP));
+            break;
+        case 0x04:
+            data = (uint64_t)*((uint32_t *)(clientmem_addr + PCM_CLIENT_IMC_PP1_TEMP));
+            break;
         default:
             syslog(LOG_ERR, "Access to register 0x%X not allowed\n", reg);
 #ifdef DEBUG_LIKWID
@@ -1226,6 +1296,7 @@ static int getBusFromSocketByNameDevid(const uint32_t socket, PciDevice* pcidev,
                                 {
                                     memset(buff, '\0', tmplen*sizeof(char));
                                     ret = fread(buff, sizeof(char), tmplen-1, fp);
+                                    fclose(fp);
                                     numa_node = atoi(buff);
                                     if (numa_node < 0)
                                     {
@@ -1453,11 +1524,11 @@ static int getBusFromSocketByNameDevidNode(const uint32_t socket, PciDevice* pci
                 if (ret > 0)
                     (*filepath)[ret] = '\0';
             }
-            fclose(fpipe);
+            pclose(fpipe);
             return socket;
         }
     }
-    fclose(fpipe);
+    pclose(fpipe);
     return -1;
 }
 
@@ -1705,6 +1776,7 @@ stop_daemon(void)
 
     free(filepath);
     closelog();
+    close_accessdaemon();
     exit(EXIT_SUCCESS);
 }
 
@@ -1862,6 +1934,11 @@ int main(void)
                     allowed = allowed_sandybridge;
                     isClientMem = 1;
                 }
+                else if (model == ICELAKE1 || model == ICELAKE2)
+                {
+                    allowed = allowed_icl;
+                    isClientMem = 1;
+                }
                 else if (model == BROADWELL_D)
                 {
                     allowed = allowed_sandybridge;
@@ -1887,6 +1964,10 @@ int main(void)
                     allowedPci = allowed_pci_skx;
                     isPCI64 = 1;
                 }
+                else if (model == ICELAKEX1 || model == ICELAKEX2)
+                {
+                    allowed = allowed_icl;
+                }
                 else if ((model == ATOM_SILVERMONT_C) ||
                          (model == ATOM_SILVERMONT_E) ||
                          (model == ATOM_SILVERMONT_Z1) ||
@@ -1915,7 +1996,20 @@ int main(void)
                 allowed = allowed_amd16;
                 break;
             case ZEN_FAMILY:
-                allowed = allowed_amd17;
+                if (model == ZEN2_RYZEN || model == ZEN2_RYZEN2)
+                {
+                    allowed = allowed_amd17_zen2;
+                }
+                else
+                {
+                    allowed = allowed_amd17;
+                }
+                break;
+            case ZEN3_FAMILY:
+                if (model == ZEN3_RYZEN || model == ZEN3_RYZEN2)
+                {
+                    allowed = allowed_amd17_zen2;
+                }
                 break;
             default:
                 syslog(LOG_ERR, "ERROR - [%s:%d] - Unsupported processor. Exiting!  \n",
@@ -1989,7 +2083,7 @@ int main(void)
          * NOTICE: This assumes consecutive processor Ids! */
         for ( uint32_t i=0; i < avail_cpus; i++ )
         {
-            sprintf(msr_file_name,"/dev/cpu/%d/msr",i);
+            snprintf(msr_file_name, MAX_PATH_LENGTH-1, "/dev/cpu/%d/msr", i);
             FD_MSR[i] = open(msr_file_name, O_RDWR);
 
             if ( FD_MSR[i] < 0 )
@@ -2170,4 +2264,3 @@ LOOP:
     /* never reached */
     return EXIT_SUCCESS;
 }
-
