@@ -5,14 +5,14 @@
  *
  *      Description:  Implementation of affinity module.
  *
- *      Version:   5.0
- *      Released:  10.11.2019
+ *      Version:   5.1.0
+ *      Released:  20.11.2020
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -61,6 +61,12 @@ int *affinity_thread2socket_lookup = NULL;
 int *affinity_thread2numa_lookup = NULL;
 int *affinity_thread2sharedl3_lookup = NULL;
 
+int *socket_lock = NULL;
+int *core_lock = NULL;
+int *tile_lock = NULL;
+int *numa_lock = NULL;
+int *sharedl2_lock = NULL;
+int *sharedl3_lock = NULL;
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
 
 #define gettid() syscall(SYS_gettid)
@@ -194,6 +200,37 @@ static int create_lookups()
         affinity_thread2numa_lookup = malloc(cpuid_topology.numHWThreads * sizeof(int));
         memset(affinity_thread2numa_lookup, -1, cpuid_topology.numHWThreads*sizeof(int));
     }
+    if (!socket_lock)
+    {
+        socket_lock = malloc(cpuid_topology.numHWThreads * sizeof(int));
+        memset(socket_lock, LOCK_INIT, cpuid_topology.numHWThreads*sizeof(int));
+    }
+    if (!numa_lock)
+    {
+        numa_lock = malloc(cpuid_topology.numHWThreads * sizeof(int));
+        memset(numa_lock, LOCK_INIT, cpuid_topology.numHWThreads*sizeof(int));
+    }
+    if (!core_lock)
+    {
+        core_lock = malloc(cpuid_topology.numHWThreads * sizeof(int));
+        memset(core_lock, LOCK_INIT, cpuid_topology.numHWThreads*sizeof(int));
+    }
+    if (!tile_lock)
+    {
+        tile_lock = malloc(cpuid_topology.numHWThreads * sizeof(int));
+        memset(tile_lock, LOCK_INIT, cpuid_topology.numHWThreads*sizeof(int));
+    }
+    if (!sharedl2_lock)
+    {
+        sharedl2_lock = malloc(cpuid_topology.numHWThreads * sizeof(int));
+        memset(sharedl2_lock, LOCK_INIT, cpuid_topology.numHWThreads*sizeof(int));
+    }
+    if (!sharedl3_lock)
+    {
+        sharedl3_lock = malloc(cpuid_topology.numHWThreads * sizeof(int));
+        memset(sharedl3_lock, LOCK_INIT, cpuid_topology.numHWThreads*sizeof(int));
+    }
+
 
     int num_pu = cpuid_topology.numHWThreads;
     if (cpuid_topology.numCacheLevels == 0)
@@ -411,6 +448,7 @@ affinity_init()
             offset = 0;
             for ( int j=0; j < (int)ceil((double)(numberOfNumaDomains)/numberOfSocketDomains); j++ )
             {
+                if (subCounter >= numberOfNumaDomains) continue;
                 domains[currentDomain + subCounter].numberOfProcessors =
                                 numa_info.nodes[subCounter].numberOfProcessors;
 
@@ -434,16 +472,10 @@ affinity_init()
                             bdata(domains[currentDomain + subCounter].tag));
                     return;
                 }
-                if (offset >= cpuid_topology.numCoresPerSocket*cpuid_topology.numSockets)
+                for (int k = 0; k < domains[currentDomain + subCounter].numberOfProcessors; k++)
                 {
-                    continue;
+                    domains[currentDomain + subCounter].processorList[k] = numa_info.nodes[subCounter].processors[k];
                 }
-                tmp = treeFillNextEntries(cpuid_topology.topologyTree,
-                                          domains[currentDomain + subCounter].processorList, 0,
-                                          i, offset, domains[currentDomain + subCounter].numberOfCores,
-                                          domains[currentDomain + subCounter].numberOfProcessors);
-                domains[currentDomain + subCounter].numberOfProcessors = tmp;
-                offset += domains[currentDomain + subCounter].numberOfCores;
                 subCounter++;
             }
         }
@@ -543,6 +575,37 @@ affinity_finalize()
         free(affinity_thread2numa_lookup);
         affinity_thread2numa_lookup = NULL;
     }
+    if (socket_lock)
+    {
+        free(socket_lock);
+        socket_lock = NULL;
+    }
+    if (numa_lock)
+    {
+        free(numa_lock);
+        numa_lock = NULL;
+    }
+    if (tile_lock)
+    {
+        free(tile_lock);
+        tile_lock = NULL;
+    }
+    if (core_lock)
+    {
+        free(core_lock);
+        core_lock = NULL;
+    }
+    if (sharedl2_lock)
+    {
+        free(sharedl2_lock);
+        sharedl2_lock = NULL;
+    }
+    if (sharedl3_lock)
+    {
+        free(sharedl3_lock);
+        sharedl3_lock = NULL;
+    }
+
 
     affinityDomains.domains = NULL;
     affinity_numberOfDomains = 0;
@@ -661,4 +724,3 @@ get_affinityDomains(void)
 {
     return &affinityDomains;
 }
-
