@@ -6,13 +6,13 @@
  *
  *      Description:  A application to set the CPU frequency of CPU cores and domains.
  *
- *      Version:   5.0
- *      Released:  10.11.2019
+ *      Version:   5.1.0
+ *      Released:  20.11.2020
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -34,7 +34,7 @@ package.path = '<INSTALLED_PREFIX>/share/lua/?.lua;' .. package.path
 local likwid = require("likwid")
 
 print_stdout = print
-print_stderr = function(...) for k,v in pairs({...}) do io.stderr:write(v .. "\n") end end
+print_stderr = function(...) for k,v in pairs({...}) do io.stderr:write(tostring(v) .. "\n") end end
 
 sys_base_path = "/sys/devices/system/cpu"
 set_command = "<INSTALLED_PREFIX>/sbin/likwid-setFreq"
@@ -120,6 +120,31 @@ function valid_freq(freq, freq_list, turbofreq)
     return valid_freq
 end
 
+function get_base_freq()
+    freq = nil
+    f = io.open("/sys/devices/system/cpu/cpu0/cpufreq/base_frequency", "r")
+    if f ~= nil then
+        out = f:read("*a")
+        freq = tonumber(out)
+        f:close()
+        return freq
+    end
+    f = io.open("/sys/devices/system/cpu/cpu0/cpufreq/bios_limit", "r")
+    if f ~= nil then
+        out = f:read("*a")
+        freq = tonumber(out)
+        f:close()
+        return freq
+    end
+    f = io.open("/proc/cpuinfo", "r")
+    if f ~= nil then
+        out = f:read("*a"):match("cpu MHz%s+:%s+(%d+.%d+)")
+        freq = tonumber(out)
+        f:close()
+    end
+    return freq*1E6
+end
+
 verbosity = 0
 governor = nil
 frequency = nil
@@ -136,6 +161,7 @@ do_ureset = false
 set_turbo = false
 turbo = 0
 driver = nil
+base_freq = nil
 
 if #arg == 0 then
     usage()
@@ -207,7 +233,7 @@ cpuinfo = likwid.getCpuInfo()
 topo = likwid.getCpuTopology()
 affinity = likwid.getAffinityInfo()
 if not domain or domain == "N" then
-    domain = "N:0-" .. tostring(topo["numHWThreads"]-1)
+    domain = "N:0-" .. tostring(topo["activeHWThreads"]-1)
 end
 if domain:match("[SCM]%d") then
     for i, dom in pairs(affinity["domains"]) do
@@ -221,34 +247,44 @@ socklist = {}
 numthreads, cpulist = likwid.cpustr_to_cpulist(domain)
 for i, dom in pairs(affinity["domains"]) do
     if dom["tag"]:match("S%d") then
-        for k, d in pairs(dom["processorList"]) do
-            local found = false
-            for j, c in pairs(cpulist) do
-                if c == d then
-
-                    found = true
-                    break
+        if #dom["processorList"] > 0 then
+            for k, d in pairs(dom["processorList"]) do
+                local found = false
+                for j, c in pairs(cpulist) do
+                    if c == d then
+                        found = true
+                        break
+                    end
                 end
-            end
-            if found then
-                s = tonumber(dom["tag"]:match("S(%d)"))
-                found = false
-                for j, c in pairs(socklist) do
-                    if c == s then found = true end
-                end
-                if not found then
-                    table.insert(socklist, s)
+                if found then
+                    s = tonumber(dom["tag"]:match("S(%d)"))
+                    found = false
+                    for j, c in pairs(socklist) do
+                        if c == s then found = true end
+                    end
+                    if not found then
+                        table.insert(socklist, s)
+                    end
                 end
             end
         end
     end
 end
 
+if cpuinfo["clock"] > 0 then
+    base_freq = cpuinfo["clock"] * 1.E-03
+else
+    base_freq = get_base_freq()
+end
+if not base_freq then
+    base_freq = likwid.getCpuClock() * 1.E-03
+end
+
 likwid.initFreq()
 driver = likwid.getFreqDriver(cpulist[1])
 
 if verbosity == 3 then
-    print_stdout(string.format("DEBUG: Given CPU expression expands to %d CPU cores:", numthreads))
+    print_stdout(string.format("DEBUG: Given CPU expression expands to %d HW Threads:", numthreads))
     local str = "DEBUG: " .. tostring(cpulist[1])
     for i=2, numthreads  do
         str = str .. "," .. tostring(cpulist[i])
@@ -308,7 +344,7 @@ if printCurFreq then
         t = tonumber(likwid.getTurbo(cpulist[i]));
         if gov and freq and min and max and t >= 0 then
             processed = processed + 1
-            table.insert(str, string.format("CPU %d: governor %12s min/cur/max %s/%s/%s GHz Turbo %d",cpulist[i], gov, round(min), round(freq), round(max), t))
+            table.insert(str, string.format("HWThread %d: governor %12s min/cur/max %s/%s/%s GHz Turbo %d",cpulist[i], gov, round(min), round(freq), round(max), t))
         end
     end
     table.insert(str, "")
@@ -428,6 +464,26 @@ if min_u_freq and max_u_freq and max_u_freq < min_u_freq then
     os.exit(1)
 end
 
+if max_freq and (likwid.getTurbo(cpulist[1]) == 1 or (set_turbo and turbo == 1)) and tonumber(max_freq) < base_freq then
+    print_stderr("ERROR: Setting maximal CPU frequency below base CPU frequency with activated Turbo mode is not supported.")
+    likwid.finalizeFreq()
+    os.exit(1)
+end
+if (likwid.getTurbo(cpulist[1]) == 0 and not set_turbo) then
+    local do_exit = false
+    if max_freq and tonumber(max_freq) > base_freq then
+        print_stderr("ERROR: Setting maximal CPU frequency above base CPU frequency with deactivated Turbo mode is not supported.")
+        do_exit = true
+    elseif min_freq and tonumber(min_freq) > base_freq then
+        print_stderr("ERROR: Setting minimal CPU frequency above base CPU frequency with deactivated Turbo mode is not supported.")
+        do_exit = true
+    end
+    if do_exit then
+        likwid.finalizeFreq()
+        os.exit(1)
+    end
+end
+
 
 local availfreqs = likwid.getAvailFreq(cpulist[1])
 if driver == "intel_pstate" then
