@@ -9,13 +9,13 @@
  *                  Allows to turn on and off the Hardware prefetcher
  *                  available.
  *
- *      Version:   5.0
- *      Released:  10.11.2019
+ *      Version:   5.1.0
+ *      Released:  20.11.2020
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -164,7 +164,8 @@ cpuFeatures_update(int cpu)
              (cpuid_info.model == SKYLAKE2) ||
              (cpuid_info.model == SKYLAKEX) ||
              (cpuid_info.model == KABYLAKE1) ||
-             (cpuid_info.model == KABYLAKE2))
+             (cpuid_info.model == KABYLAKE2) ||
+             (cpuid_info.model == CANNONLAKE))
     {
         TEST_FLAG_INV(FEAT_TURBO_MODE,38);
     }
@@ -193,7 +194,12 @@ cpuFeatures_update(int cpu)
             (cpuid_info.model == SKYLAKEX) ||
             (cpuid_info.model == KABYLAKE1) ||
             (cpuid_info.model == KABYLAKE2) ||
-            (cpuid_info.model == ATOM_SILVERMONT_GOLD))
+            (cpuid_info.model == ICELAKE1) ||
+            (cpuid_info.model == ICELAKE2) ||
+            (cpuid_info.model == ICELAKEX1) ||
+            (cpuid_info.model == ICELAKEX2) ||
+            (cpuid_info.model == ATOM_SILVERMONT_GOLD) ||
+            (cpuid_info.model == CANNONLAKE))
     {
         ret = HPMread(cpu, MSR_DEV, MSR_PREFETCH_ENABLE, &flags);
         if (ret != 0)
@@ -367,6 +373,11 @@ cpuFeatures_enable(int cpu, CpuFeature type, int print)
             (cpuid_info.model == SKYLAKEX) ||
             (cpuid_info.model == KABYLAKE1) ||
             (cpuid_info.model == KABYLAKE2) ||
+            (cpuid_info.model == CANNONLAKE) ||
+            (cpuid_info.model == ICELAKE1) ||
+            (cpuid_info.model == ICELAKE2) ||
+            (cpuid_info.model == ICELAKEX1) ||
+            (cpuid_info.model == ICELAKEX2) ||
             (cpuid_info.model == ATOM_SILVERMONT_GOLD))
     {
         reg = MSR_PREFETCH_ENABLE;
@@ -523,6 +534,11 @@ cpuFeatures_disable(int cpu, CpuFeature type, int print)
             (cpuid_info.model == SKYLAKEX) ||
             (cpuid_info.model == KABYLAKE1) ||
             (cpuid_info.model == KABYLAKE2) ||
+            (cpuid_info.model == CANNONLAKE) ||
+            (cpuid_info.model == ICELAKE1) ||
+            (cpuid_info.model == ICELAKE2) ||
+            (cpuid_info.model == ICELAKEX1) ||
+            (cpuid_info.model == ICELAKEX2) ||
             (cpuid_info.model == ATOM_SILVERMONT_GOLD))
     {
         reg = MSR_PREFETCH_ENABLE;
