@@ -5,14 +5,14 @@
  *
  *      Description:  Header File affinity Module
  *
- *      Version:   5.0
- *      Released:  10.11.2019
+ *      Version:   5.1.0
+ *      Released:  20.11.2020
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Gruber (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -34,12 +34,12 @@
 #include <types.h>
 #include <likwid.h>
 
-int socket_lock[MAX_NUM_NODES];
-int core_lock[MAX_NUM_THREADS];
-int tile_lock[MAX_NUM_THREADS];
-int numa_lock[MAX_NUM_THREADS];
-int sharedl2_lock[MAX_NUM_THREADS];
-int sharedl3_lock[MAX_NUM_THREADS];
+extern int *socket_lock;
+extern int *core_lock;
+extern int *tile_lock;
+extern int *numa_lock;
+extern int *sharedl2_lock;
+extern int *sharedl3_lock;
 
 extern AffinityDomains affinityDomains;
 
