@@ -6,13 +6,13 @@
  *      Description:  Header File NUMA module for internal use. External functions are
  *                    defined in likwid.h
  *
- *      Version:   5.0
- *      Released:  10.11.2019
+ *      Version:   5.1.0
+ *      Released:  20.11.2020
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -38,11 +38,16 @@
 #include <likwid.h>
 #include <numa_hwloc.h>
 #include <numa_proc.h>
+#include <numa_virtual.h>
 
 extern int numaInitialized;
 
 extern int str2int(const char* str);
 
+extern uint64_t proc_getFreeSysMem(void);
+
+extern uint64_t proc_getTotalSysMem(void);
+
 struct numa_functions {
     int (*numa_init) (void);
     void (*numa_setInterleaved) (const int*, int);
