@@ -5,13 +5,13 @@
  *
  *      Description:  Header File of nvmon module (CUPTI backend).
  *
- *      Version:   5.0
- *      Released:  10.11.2019
+ *      Version:   5.1.0
+ *      Released:  20.11.2020
  *
  *      Author:   Thomas Gruber (tg), thomas.gruber@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -994,6 +994,7 @@ NvmonFunctions nvmon_cupti_functions = {
     .addEvents = nvmon_cupti_addEventSets,
     .setupCounters = nvmon_cupti_setupCounters,
     .startCounters = nvmon_cupti_startCounters,
+    .stopCounters = nvmon_cupti_stopCounters,
     .readCounters = nvmon_cupti_readCounters,
 };
 
