@@ -5,13 +5,13 @@
  *
  *      Description:  Header File of perfmon module for perf_event kernel interface.
  *
- *      Version:   5.0
- *      Released:  10.11.2019
+ *      Version:   5.1.0
+ *      Released:  20.11.2020
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -46,6 +46,7 @@ static int active_cpus = 0;
 static int paranoid_level = -1;
 static int informed_paranoid = 0;
 static int running_group = -1;
+static int perf_event_num_cpus = 0;
 
 static long
 perf_event_open(struct perf_event_attr *hw_event, pid_t pid,
@@ -120,38 +121,11 @@ int perfmon_init_perfevent(int cpu_id)
         memset(cpu_event_fds[cpu_id], -1, perfmon_numCounters * sizeof(int));
         active_cpus += 1;
     }
+    perf_event_num_cpus = cpuid_topology.numHWThreads;
     return 0;
 }
 
-int perf_fixed_setup(struct perf_event_attr *attr, RegisterIndex index, PerfmonEvent *event)
-{
-    int ret = -1;
-    attr->type = PERF_TYPE_HARDWARE;
-    attr->exclude_kernel = 1;
-    attr->exclude_hv = 1;
-    attr->disabled = 1;
-    attr->inherit = 1;
-    //attr->exclusive = 1;
-    if (strcmp(event->name, "INSTR_RETIRED_ANY") == 0)
-    {
-        attr->config = PERF_COUNT_HW_INSTRUCTIONS;
-        ret = 0;
-    }
-    if (strcmp(event->name, "CPU_CLK_UNHALTED_CORE") == 0)
-    {
-        attr->config = PERF_COUNT_HW_CPU_CYCLES;
-        ret = 0;
-    }
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,3,0)
-    if (strcmp(event->name, "CPU_CLK_UNHALTED_REF") == 0)
-    {
-        attr->config = PERF_COUNT_HW_REF_CPU_CYCLES;
-        ret = 0;
-    }
-#endif
 
-    return ret;
-}
 
 typedef enum {
     PERF_EVENT_INVAL_REG = 0,
@@ -177,6 +151,12 @@ static char* perfEventOptionNames[] = {
     [EVENT_OPTION_OCCUPANCY_FILTER] = "occ_band0",
     [EVENT_OPTION_OCCUPANCY_EDGE] = "occ_edge",
     [EVENT_OPTION_OCCUPANCY_INVERT] = "occ_inv",
+#ifdef _ARCH_PPC
+    [EVENT_OPTION_GENERIC_CONFIG] = "pmcxsel",
+#else
+    [EVENT_OPTION_GENERIC_CONFIG] = "event",
+#endif
+    [EVENT_OPTION_GENERIC_UMASK] = "umask",
 #ifdef _ARCH_PPC
     [EVENT_OPTION_PMC] = "pmc",
     [EVENT_OPTION_PMCXSEL] = "pmcxsel",
@@ -263,18 +243,140 @@ uint64_t create_mask(uint32_t value, int start, int end)
     return 0x0ULL;
 }
 
+int perf_fixed_setup(struct perf_event_attr *attr, RegisterIndex index, PerfmonEvent *event)
+{
+    int ret = -1;
+    attr->type = PERF_TYPE_HARDWARE;
+    attr->disabled = 1;
+    attr->inherit = 1;
+    if (translate_types[FIXED] != NULL &&
+        strcmp(translate_types[PMC], translate_types[FIXED]) == 0)
+    {
+        attr->exclude_kernel = 1;
+        attr->exclude_hv = 1;
+        if (strcmp(event->name, "INSTR_RETIRED_ANY") == 0)
+        {
+            attr->config = PERF_COUNT_HW_INSTRUCTIONS;
+            ret = 0;
+        }
+        if (strcmp(event->name, "CPU_CLK_UNHALTED_CORE") == 0 ||
+            strcmp(event->name, "ACTUAL_CPU_CLOCK") == 0 ||
+            strcmp(event->name, "APERF") == 0)
+        {
+            attr->config = PERF_COUNT_HW_CPU_CYCLES;
+            ret = 0;
+        }
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,3,0)
+        if (strcmp(event->name, "CPU_CLK_UNHALTED_REF") == 0)
+        {
+            attr->config = PERF_COUNT_HW_REF_CPU_CYCLES;
+            ret = 0;
+        }
+#endif
+    }
+    else
+    {
+        char checkfolder[1024];
+        int perf_type = PERF_TYPE_HARDWARE;
+        int start = 0, end = 0;
+        PERF_EVENT_PMC_OPT_REGS reg = PERF_EVENT_INVAL_REG;
+        int err = snprintf(checkfolder, 1023, "%s/type", translate_types[FIXED]);
+        if (err > 0)
+        {
+            checkfolder[err] = '\0';
+        }
+        FILE* fp = fopen(checkfolder, "r");
+        if (fp != NULL)
+        {
+            ret = fread(checkfolder, sizeof(char), 1024, fp);
+            fclose(fp);
+            perf_type = atoi(checkfolder);
+            if (perf_type >= 0)
+            {
+                attr->type = perf_type;
+                getEventOptionConfig(translate_types[FIXED], EVENT_OPTION_GENERIC_CONFIG, &reg, &start, &end);
+                switch(reg)
+                {
+                    case PERF_EVENT_CONFIG_REG:
+                        attr->config |= create_mask(event->eventId, start, end);
+                        ret = 0;
+                        break;
+                    case PERF_EVENT_CONFIG1_REG:
+                        attr->config1 |= create_mask(event->eventId, start, end);
+                        ret = 0;
+                        break;
+                    case PERF_EVENT_CONFIG2_REG:
+                        attr->config2 |= create_mask(event->eventId, start, end);
+                        ret = 0;
+                        break;
+                    default:
+                        ret = -1;
+                        break;
+                }
+            }
+        }
+    }
+
+    return ret;
+}
+
 int perf_pmc_setup(struct perf_event_attr *attr, RegisterIndex index, PerfmonEvent *event)
 {
     uint64_t offcore_flags = 0x0ULL;
     PERF_EVENT_PMC_OPT_REGS reg = PERF_EVENT_INVAL_REG;
     int start = 0, end = -1;
     attr->type = PERF_TYPE_RAW;
-    attr->config = (event->umask<<8) + event->eventId;
+    //attr->config = (event->umask<<8) + event->eventId;
     attr->exclude_kernel = 1;
     attr->exclude_hv = 1;
     attr->disabled = 1;
     attr->inherit = 1;
     //attr->exclusive = 1;
+#if defined(__ARM_ARCH_8A) || defined(__ARM_ARCH_7A__)
+    if (cpuid_info.vendor == FUJITSU_ARM && cpuid_info.part == FUJITSU_A64FX)
+    {
+        reg = PERF_EVENT_CONFIG_REG;
+        start = 0;
+        end = 31;
+    }
+    else
+    {
+#endif
+        getEventOptionConfig(translate_types[PMC], EVENT_OPTION_GENERIC_CONFIG, &reg, &start, &end);
+#if defined(__ARM_ARCH_8A) || defined(__ARM_ARCH_7A__)
+    }
+#endif
+    switch(reg)
+    {
+        case PERF_EVENT_CONFIG_REG:
+            attr->config |= create_mask(event->eventId, start, end);
+            break;
+        case PERF_EVENT_CONFIG1_REG:
+            attr->config1 |= create_mask(event->eventId, start, end);
+            break;
+        case PERF_EVENT_CONFIG2_REG:
+            attr->config2 |= create_mask(event->eventId, start, end);
+            break;
+    }
+#ifdef _ARCH_PPC
+    reg = PERF_EVENT_CONFIG_REG;
+    start = 8;
+    end = 15;
+#else
+    getEventOptionConfig(translate_types[PMC], EVENT_OPTION_GENERIC_UMASK, &reg, &start, &end);
+#endif
+    switch(reg)
+    {
+        case PERF_EVENT_CONFIG_REG:
+            attr->config |= create_mask(event->umask, start, end);
+            break;
+        case PERF_EVENT_CONFIG1_REG:
+            attr->config1 |= create_mask(event->umask, start, end);
+            break;
+        case PERF_EVENT_CONFIG2_REG:
+            attr->config2 |= create_mask(event->umask, start, end);
+            break;
+    }
     if (event->numberOfOptions > 0)
     {
         for(int j = 0; j < event->numberOfOptions; j++)
@@ -291,7 +393,7 @@ int perf_pmc_setup(struct perf_event_attr *attr, RegisterIndex index, PerfmonEve
                 case EVENT_OPTION_INVERT:
                 case EVENT_OPTION_IN_TRANS:
                 case EVENT_OPTION_IN_TRANS_ABORT:
-                    getEventOptionConfig("/sys/devices/cpu", event->options[j].type, &reg, &start, &end);
+                    getEventOptionConfig(translate_types[PMC], event->options[j].type, &reg, &start, &end);
                     switch(reg)
                     {
                         case PERF_EVENT_CONFIG_REG:
@@ -328,7 +430,7 @@ int perf_pmc_setup(struct perf_event_attr *attr, RegisterIndex index, PerfmonEve
         {
             offcore_flags = (1ULL<<event->cfgBits)|(1ULL<<event->cmask);
         }
-        getEventOptionConfig("/sys/devices/cpu", EVENT_OPTION_MATCH0, &reg, &start, &end);
+        getEventOptionConfig(translate_types[PMC], EVENT_OPTION_MATCH0, &reg, &start, &end);
         switch(reg)
         {
             case PERF_EVENT_CONFIG_REG:
@@ -343,7 +445,7 @@ int perf_pmc_setup(struct perf_event_attr *attr, RegisterIndex index, PerfmonEve
         }
     }
 #ifdef _ARCH_PPC
-    getEventOptionConfig("/sys/devices/cpu", EVENT_OPTION_PMC, &reg, &start, &end);
+    getEventOptionConfig(translate_types[PMC], EVENT_OPTION_PMC, &reg, &start, &end);
     switch(reg)
     {
         case PERF_EVENT_CONFIG_REG:
@@ -371,7 +473,7 @@ int perf_uncore_setup(struct perf_event_attr *attr, RegisterType type, PerfmonEv
     int perf_type = 0;
     PERF_EVENT_PMC_OPT_REGS reg = PERF_EVENT_INVAL_REG;
     int start = 0, end = -1;
-    if (paranoid_level > 0)
+    if (paranoid_level > 0 && getuid() != 0)
     {
         return 1;
     }
@@ -392,6 +494,7 @@ int perf_uncore_setup(struct perf_event_attr *attr, RegisterType type, PerfmonEv
             return 1;
         }
     }
+    DEBUG_PRINT(DEBUGLEV_DEVELOP, Get information for uncore counters from folder %s, checkfolder);
     ret = sprintf(&(checkfolder[ret]), "/type");
     fp = fopen(checkfolder, "r");
     if (fp == NULL)
@@ -402,9 +505,36 @@ int perf_uncore_setup(struct perf_event_attr *attr, RegisterType type, PerfmonEv
     perf_type = atoi(checkfolder);
     fclose(fp);
     attr->type = perf_type;
-    attr->config = (event->umask<<8) + event->eventId;
     attr->disabled = 1;
     attr->inherit = 1;
+    //attr->config = (event->umask<<8) + event->eventId;
+    getEventOptionConfig(translate_types[type], EVENT_OPTION_GENERIC_CONFIG, &reg, &start, &end);
+    switch(reg)
+    {
+        case PERF_EVENT_CONFIG_REG:
+            attr->config |= create_mask(event->eventId, start, end);
+            break;
+        case PERF_EVENT_CONFIG1_REG:
+            attr->config1 |= create_mask(event->eventId, start, end);
+            break;
+        case PERF_EVENT_CONFIG2_REG:
+            attr->config2 |= create_mask(event->eventId, start, end);
+            break;
+    }
+    getEventOptionConfig(translate_types[type], EVENT_OPTION_GENERIC_UMASK, &reg, &start, &end);
+    switch(reg)
+    {
+        case PERF_EVENT_CONFIG_REG:
+            attr->config |= create_mask(event->umask, start, end);
+            break;
+        case PERF_EVENT_CONFIG1_REG:
+            attr->config1 |= create_mask(event->umask, start, end);
+            break;
+        case PERF_EVENT_CONFIG2_REG:
+            attr->config2 |= create_mask(event->umask, start, end);
+            break;
+    }
+
 
     //attr->exclusive = 1;
     if (event->numberOfOptions > 0)
@@ -477,10 +607,23 @@ int perfmon_setupCountersThread_perfevent(
     {
         allflags = strtoul(getenv("LIKWID_PERF_FLAGS"), NULL, 16);
     }
+    if (groupSet->activeGroup >= 0)
+    {
+        for (int j = 0; j < perfmon_numCounters; j++)
+        {
+            if (cpu_event_fds[cpu_id][j] != -1)
+            {
+                close(cpu_event_fds[cpu_id][j]);
+                cpu_event_fds[cpu_id][j] = -1;
+            }
+        }
+    }
     for (int i=0;i < eventSet->numberOfEvents;i++)
     {
         int has_lock = 0;
+        int pmc_lock = 0;
         is_uncore = 0;
+        ret = 1;
         RegisterIndex index = eventSet->events[i].index;
         if (cpu_event_fds[cpu_id][index] != -1)
         {
@@ -505,13 +648,37 @@ int perfmon_setupCountersThread_perfevent(
                 VERBOSEPRINTREG(cpu_id, index, attr.config, SETUP_FIXED);
                 break;
             case PMC:
-                ret = perf_pmc_setup(&attr, index, event);
-                VERBOSEPRINTREG(cpu_id, index, attr.config, SETUP_PMC);
+                pmc_lock = 1;
+#if defined(__ARM_ARCH_8A)
+                if (cpuid_info.vendor == FUJITSU_ARM && cpuid_info.part == FUJITSU_A64FX)
+                {
+                    if (event->eventId == 0x3E8 ||
+                        event->eventId == 0x3E0 ||
+                        event->eventId == 0x308 ||
+                        event->eventId == 0x309 ||
+                        (event->eventId >= 0x314 &&  event->eventId <= 0x31E))
+                    {
+                        if (numa_lock[affinity_thread2numa_lookup[cpu_id]] != cpu_id)
+                        {
+                            pmc_lock = 0;
+                        }
+                    }
+                }
+#endif
+                if (pmc_lock)
+                {
+                    ret = perf_pmc_setup(&attr, index, event);
+                    VERBOSEPRINTREG(cpu_id, index, attr.config, SETUP_PMC);
+                }
                 break;
             case POWER:
-                ret = perf_uncore_setup(&attr, type, event);
+                if (socket_lock[affinity_thread2socket_lookup[cpu_id]] == cpu_id)
+                {
+                    has_lock = 1;
+                    VERBOSEPRINTREG(cpu_id, index, attr.config, SETUP_POWER);
+                    ret = perf_uncore_setup(&attr, type, event);
+                }
                 is_uncore = 1;
-                VERBOSEPRINTREG(cpu_id, index, attr.config, SETUP_POWER);
                 break;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3,5,0)
             case MBOX0:
@@ -551,6 +718,7 @@ int perfmon_setupCountersThread_perfevent(
             case CBOX26:
             case CBOX27:
             case UBOX:
+            case UBOXFIX:
             case SBOX0:
             case SBOX1:
             case SBOX2:
@@ -579,7 +747,6 @@ int perfmon_setupCountersThread_perfevent(
             case EUBOX5:
             case EUBOX6:
             case EUBOX7:
-
                 if (cpuid_info.family == ZEN_FAMILY && type == MBOX0)
                 {
                     if (numa_lock[affinity_thread2numa_lookup[cpu_id]] == cpu_id)
@@ -735,28 +902,46 @@ int perfmon_readCountersThread_perfevent(int thread_id, PerfmonEventSet* eventSe
 int perfmon_finalizeCountersThread_perfevent(int thread_id, PerfmonEventSet* eventSet)
 {
     int cpu_id = groupSet->threads[thread_id].processorId;
-    for (int i=0;i < eventSet->numberOfEvents;i++)
+    if (cpu_event_fds != NULL)
     {
-        if (eventSet->events[i].threadCounter[thread_id].init == TRUE)
+        if (cpu_event_fds[cpu_id] != NULL)
         {
-            RegisterIndex index = eventSet->events[i].index;
-            ioctl(cpu_event_fds[cpu_id][index], PERF_EVENT_IOC_DISABLE, 0);
-            ioctl(cpu_event_fds[cpu_id][index], PERF_EVENT_IOC_RESET, 0);
-            eventSet->events[i].threadCounter[thread_id].init = FALSE;
-            close(cpu_event_fds[cpu_id][index]);
-            cpu_event_fds[cpu_id][index] = -1;
+            for (int j = 0; j < perfmon_numCounters; j++)
+            {
+                if (cpu_event_fds[cpu_id][j] > 0)
+                {
+                    ioctl(cpu_event_fds[cpu_id][j], PERF_EVENT_IOC_DISABLE, 0);
+                    ioctl(cpu_event_fds[cpu_id][j], PERF_EVENT_IOC_RESET, 0);
+                    close(cpu_event_fds[cpu_id][j]);
+                    cpu_event_fds[cpu_id][j] = -1;
+                    if (j < eventSet->numberOfEvents && eventSet->events[j].threadCounter[thread_id].init == TRUE)
+                    {
+                        eventSet->events[j].threadCounter[thread_id].init = FALSE;
+                    }
+                }
+            }
+            free(cpu_event_fds[cpu_id]);
+            cpu_event_fds[cpu_id] = NULL;
+            active_cpus--;
         }
     }
-    if (cpu_event_fds[cpu_id] != NULL)
-    {
-        free(cpu_event_fds[cpu_id]);
-        cpu_event_fds[cpu_id] = NULL;
-        active_cpus--;
-    }
-    if (active_cpus == 0)
-    {
-        free(cpu_event_fds);
-        cpu_event_fds = NULL;
-    }
     return 0;
 }
+
+void __attribute__((destructor (101))) close_perfmon_perfevent(void)
+{
+    if (cpu_event_fds != NULL)
+    {
+        for (int i = 0; i < perf_event_num_cpus; i++)
+        {
+            if (cpu_event_fds[i] != NULL)
+            {
+                free(cpu_event_fds[i]);
+                cpu_event_fds[i] = NULL;
+                active_cpus--;
+            }
+        }
+        free(cpu_event_fds);
+        cpu_event_fds = NULL;
+    }
+}
