@@ -5,14 +5,14 @@
  *
  *      Description:  Header File of perfmon module for Intel Skylake.
  *
- *      Version:   5.0
- *      Released:  10.11.2019
+ *      Version:   5.1.0
+ *      Released:  20.11.2020
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -38,6 +38,7 @@
 #include <limits.h>
 #include <topology.h>
 #include <access.h>
+#include <voltage.h>
 #include <linux/version.h>
 
 static int perfmon_numCountersSkylake = NUM_COUNTERS_SKYLAKE;
@@ -75,25 +76,40 @@ int perfmon_init_skylake(int cpu_id)
     {
         ERROR_PRINT(Cannot zero MSR_PEBS_FRONTEND (0x%X), MSR_PEBS_FRONTEND);
     }
-    if (cpuid_info.model == SKYLAKEX)
+    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] == cpu_id)
     {
-        skylake_cbox_setup = skx_cbox_setup;
-        skl_did_cbox_check = 1;
-    }
-    else if ((cpuid_info.model == SKYLAKE1 || cpuid_info.model == SKYLAKE2) &&
-             socket_lock[affinity_thread2socket_lookup[cpu_id]] == cpu_id &&
-             skl_did_cbox_check == 0)
-    {
-        uint64_t data = 0x0ULL;
-        ret = HPMwrite(cpu_id, MSR_DEV, MSR_V4_C0_PERF_CTRL0, 0x0ULL);
-        ret += HPMread(cpu_id, MSR_DEV, MSR_V4_UNC_PERF_GLOBAL_CTRL, &data);
-        ret += HPMwrite(cpu_id, MSR_DEV, MSR_V4_UNC_PERF_GLOBAL_CTRL, 0x0ULL);
-        ret += HPMread(cpu_id, MSR_DEV, MSR_V4_C0_PERF_CTRL0, &data);
-        if ((ret == 0) && (data == 0x0ULL))
-            skylake_cbox_setup = skl_cbox_setup;
-        else
-            skylake_cbox_setup = skl_cbox_nosetup;
-        skl_did_cbox_check = 1;
+        switch (cpuid_info.model)
+        {
+            case SKYLAKEX:
+                if (skl_did_cbox_check == 0)
+                {
+                    skylake_cbox_setup = skx_cbox_setup;
+                    skl_did_cbox_check = 1;
+                }
+                break;
+            case SKYLAKE1:
+            case SKYLAKE2:
+            case KABYLAKE1:
+            case KABYLAKE2:
+            case CANNONLAKE:
+                if (skl_did_cbox_check == 0)
+                {
+                    uint64_t data = 0x0ULL;
+                    ret = HPMwrite(cpu_id, MSR_DEV, MSR_V4_C0_PERF_CTRL0, 0x0ULL);
+                    ret += HPMread(cpu_id, MSR_DEV, MSR_V4_UNC_PERF_GLOBAL_CTRL, &data);
+                    ret += HPMwrite(cpu_id, MSR_DEV, MSR_V4_UNC_PERF_GLOBAL_CTRL, 0x0ULL);
+                    ret += HPMread(cpu_id, MSR_DEV, MSR_V4_C0_PERF_CTRL0, &data);
+                    if ((ret == 0) && (data == 0x0ULL))
+                        skylake_cbox_setup = skl_cbox_setup;
+                    else
+                        skylake_cbox_setup = skl_cbox_nosetup;
+                    skl_did_cbox_check = 1;
+                }
+                break;
+            default:
+                skylake_cbox_setup = skl_cbox_nosetup;
+                skl_did_cbox_check = 1;
+        }
     }
     return 0;
 }
@@ -370,7 +386,7 @@ int skx_cbox_setup(int cpu_id, RegisterIndex index, PerfmonEvent *event)
                     set_state_all = 0;
                     break;
                 case EVENT_OPTION_TID:
-                    filter_flags0 |= (extractBitField(event->options[j].value,8,0));
+                    filter_flags0 |= (extractBitField(event->options[j].value,9,0));
                     flags |= (1ULL<<19);
                     break;
                 case EVENT_OPTION_MATCH0:
@@ -1311,6 +1327,10 @@ int perfmon_stopCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet)
                 case THERMAL:
                     CHECK_TEMP_READ_ERROR(thermal_read(cpu_id,(uint32_t*)&counter_result));
                     break;
+                
+                case VOLTAGE:
+                    CHECK_TEMP_READ_ERROR(voltage_read(cpu_id, &counter_result));
+                    break;
 
                 case UBOXFIX:
                     if (haveLock)
@@ -1383,6 +1403,7 @@ int perfmon_stopCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet)
                     }
                     break;
                 case MBOX0:
+                case MBOX0TMP:
                     if (haveLock)
                     {
                         if (!cpuid_info.supportClientmem)
@@ -1456,7 +1477,6 @@ int perfmon_stopCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet)
             }
             *current = field64(counter_result, 0, box_map[type].regWidth);
         }
-        eventSet->events[i].threadCounter[thread_id].init = FALSE;
     }
     if ((haveLock) && MEASURE_UNCORE(eventSet))
     {
@@ -1543,6 +1563,11 @@ int perfmon_readCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet)
                     CHECK_TEMP_READ_ERROR(thermal_read(cpu_id,(uint32_t*)&counter_result));
                     eventSet->events[i].threadCounter[thread_id].counterData = field64(counter_result, 0, box_map[type].regWidth);
                     break;
+                
+                case VOLTAGE:
+                    CHECK_TEMP_READ_ERROR(voltage_read(cpu_id, &counter_result));
+                    eventSet->events[i].threadCounter[thread_id].counterData = field64(counter_result, 0, box_map[type].regWidth);
+                    break;
 
                 case UBOXFIX:
                 case WBOX0FIX:
@@ -1608,6 +1633,7 @@ int perfmon_readCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet)
                     }
                     break;
                 case MBOX0:
+                case MBOX0TMP:
                     if (haveLock)
                     {
                         if (!cpuid_info.supportClientmem)
