@@ -5,14 +5,14 @@
  *
  *      Description:  Get NUMA topology from procfs and sysfs
  *
- *      Version:   5.0
- *      Released:  10.11.2019
+ *      Version:   5.1.0
+ *      Released:  20.11.2020
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -348,9 +348,8 @@ int proc_numa_init(void)
 
     if (get_mempolicy(NULL, NULL, 0, 0, 0) < 0 && errno == ENOSYS)
     {
-        numa_info.numberOfNodes = 0;
-        numa_info.nodes = NULL;
-        return -1;
+        /* Allocate a virtual node instead and bail out */
+        return virtual_numa_init();
     }
     /* First determine maximum number of nodes */
     //numa_info.numberOfNodes = setConfiguredNodes()+1;
