@@ -5,14 +5,14 @@
  *
  *      Description:  Handler for performance groups and event sets
  *
- *      Version:   5.0
- *      Released:  10.11.2019
+ *      Version:   5.1.0
+ *      Released:  20.11.2020
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Gruber (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -598,6 +598,7 @@ int perfgroup_customGroup(const char* eventStr, GroupInfo* ginfo)
     int has_fix0 = 0;
     int has_fix1 = 0;
     int has_fix2 = 0;
+    int has_fix3 = 0;
     int gpu_events = 0;
     ginfo->shortinfo = NULL;
     ginfo->nevents = 0;
@@ -613,6 +614,7 @@ int perfgroup_customGroup(const char* eventStr, GroupInfo* ginfo)
     bstring fix0 = bformat("FIXC0");
     bstring fix1 = bformat("FIXC1");
     bstring fix2 = bformat("FIXC2");
+    bstring fix3 = bformat("FIXC3");
 #endif
 #ifdef _ARCH_PPC
     bstring fix0 = bformat("PMC4");
@@ -677,6 +679,14 @@ int perfgroup_customGroup(const char* eventStr, GroupInfo* ginfo)
         {
             ginfo->nevents++;
         }
+        if (binstr(eventBstr, 0, fix3) > 0)
+        {
+            has_fix3 = 1;
+        }
+        else
+        {
+            ginfo->nevents++;
+        }
 #endif
     }
     bdestroy(eventBstr);
@@ -763,6 +773,15 @@ int perfgroup_customGroup(const char* eventStr, GroupInfo* ginfo)
             sprintf(ginfo->counters[i], "%s", "FIXC2");
             i++;
         }
+        if ((!has_fix3) && cpuid_info.perf_num_fixed_ctr > 3 && 
+            (cpuid_info.model == ICELAKE1 || cpuid_info.model == ICELAKE2 || cpuid_info.model == ICELAKEX1 || cpuid_info.model == ICELAKEX2))
+        {
+            ginfo->events[i] = malloc(14 * sizeof(char));
+            ginfo->counters[i] = malloc(6 * sizeof(char));
+            sprintf(ginfo->events[i], "%s", "TOPDOWN_SLOTS");
+            sprintf(ginfo->counters[i], "%s", "FIXC3");
+            i++;
+        }
     }
     ginfo->nevents = i;
 #endif
@@ -1596,6 +1615,7 @@ calc_metric(char* formula, CounterList* clist, double *result)
     // now we can calculate the formula
     i = calculate_infix(bdata(f), result);
     bdestroy(f);
+    bdestroy(inf);
     bdestroy(nan);
     bdestroy(zero);
     return i;
