@@ -5,14 +5,14 @@
  *
  *      Description:  Module implementing Intel RAPL interface
  *
- *      Version:   5.0
- *      Released:  10.11.2019
+ *      Version:   5.1.0
+ *      Released:  20.11.2020
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -124,6 +124,13 @@ power_init(int cpuId)
                 case SKYLAKEX:
                 case KABYLAKE1:
                 case KABYLAKE2:
+                case CANNONLAKE:
+                case TIGERLAKE1:
+                case TIGERLAKE2:
+                case ICELAKE1:
+                case ICELAKE2:
+                case ICELAKEX1:
+                case ICELAKEX2:
                     power_info.hasRAPL = 1;
                     numDomains = NUM_POWER_DOMAINS;
                     break;
@@ -142,8 +149,14 @@ power_init(int cpuId)
             }
             break;
         case ZEN_FAMILY:
+        case ZEN3_FAMILY:
             if (cpuid_info.model == ZEN_RYZEN ||
-                cpuid_info.model == ZEN2_RYZEN)
+                cpuid_info.model == ZENPLUS_RYZEN ||
+                cpuid_info.model == ZENPLUS_RYZEN2 ||
+                cpuid_info.model == ZEN2_RYZEN ||
+                cpuid_info.model == ZEN2_RYZEN2 ||
+                cpuid_info.model == ZEN3_RYZEN ||
+                cpuid_info.model == ZEN3_RYZEN2)
             {
                 cpuid_info.turbo = 0;
                 power_info.hasRAPL = 1;
@@ -162,6 +175,7 @@ power_init(int cpuId)
                     info_regs[i] = 0x0;
                 }
             }
+            break;
     }
 
     perfmon_init_maps();
@@ -658,6 +672,7 @@ power_finalize(void)
     power_info.uncoreMinFreq = 0;
     power_info.uncoreMaxFreq = 0;
     memset(power_info.domains, 0, NUM_POWER_DOMAINS*sizeof(PowerDomain));
+    power_initialized = 0;
 }
 
 PowerInfo_t
