@@ -5,14 +5,14 @@
  *
  *      Description:  Interface to the topology backends
  *
- *      Version:   5.0
- *      Released:  10.11.2019
+ *      Version:   5.1.0
+ *      Released:  20.11.2020
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -89,6 +89,8 @@ static char* xeon_phi_string = "Intel Xeon Phi (Knights Corner) Coprocessor";
 static char* xeon_phi2_string = "Intel Xeon Phi (Knights Landing) (Co)Processor";
 static char* xeon_phi3_string = "Intel Xeon Phi (Knights Mill) (Co)Processor";
 static char* icelake_str = "Intel Icelake processor";
+static char* tigerlake_str = "Intel Tigerlake processor";
+static char* icelakesp_str = "Intel Icelake SP processor";
 //static char* snowridgex_str = "Intel SnowridgeX processor";
 
 static char* barcelona_str = "AMD K10 (Barcelona) processor";
@@ -107,13 +109,17 @@ static char* athlon64_X2_g_str = "AMD Athlon64 X2 (AM2) Rev G 65nm processor";
 static char* athlon64_g_str = "AMD Athlon64 (AM2) Rev G 65nm processor";
 static char* amd_k8_str = "AMD K8 architecture";
 static char* amd_zen_str = "AMD K17 (Zen) architecture";
+static char* amd_zenplus_str = "AMD K17 (Zen+) architecture";
 static char* amd_zen2_str = "AMD K17 (Zen2) architecture";
+static char* amd_zen3_str = "AMD K19 (Zen3) architecture";
 static char* armv7l_str = "ARM 7l architecture";
 static char* armv8_str = "ARM 8 architecture";
 static char* cavium_thunderx2t99_str = "Cavium Thunder X2 (ARMv8)";
 static char* cavium_thunderx_str = "Cavium Thunder X (ARMv8)";
 static char* arm_cortex_a57 = "ARM Cortex A57 (ARMv8)";
 static char* arm_cortex_a53 = "ARM Cortex A53 (ARMv8)";
+static char* arm_neoverse_n1 = "ARM Neoverse N1";
+static char* fujitsu_a64fx = "Fujitsu A64FX";
 static char* power7_str = "POWER7 architecture";
 static char* power8_str = "POWER8 architecture";
 static char* power9_str = "POWER9 architecture";
@@ -146,9 +152,11 @@ static char* short_skylakeX = "skylakeX";
 static char* short_kabylake = "skylake";
 static char* short_cascadelakeX = "CLX";
 static char* short_cannonlake = "cannonlake";
+static char* short_tigerlake = "TGL";
 static char* short_phi = "phi";
 static char* short_phi2 = "knl";
 static char* short_icelake = "ICL";
+static char* short_icelakesp = "ICX";
 //static char* short_snowridgex = "SNR";
 
 static char* short_k8 = "k8";
@@ -157,11 +165,14 @@ static char* short_k15 = "interlagos";
 static char* short_k16 = "kabini";
 static char* short_zen = "zen";
 static char* short_zen2 = "zen2";
+static char* short_zen3 = "zen3";
 
 static char* short_arm7 = "arm7";
 static char* short_arm8 = "arm8";
 static char* short_arm8_cav_tx2 = "arm8_tx2";
 static char* short_arm8_cav_tx = "arm8_tx";
+static char* short_arm8_neo_n1 = "arm8_n1";
+static char* short_a64fx = "arm64fx";
 
 static char* short_power7 = "power7";
 static char* short_power8 = "power8";
@@ -430,9 +441,10 @@ readTopologyFile(const char* filename, cpu_set_t cpuSet)
             }
             else if (strcmp(field, "osname") == 0)
             {
-                strcpy(value,&(line[strlen(structure)+strlen(field)+4]));
-                cpuid_info.osname = (char*) malloc((strlen(value)+1) * sizeof(char));
-                strncpy(cpuid_info.osname, value, strlen(value));
+                strncpy(value,&(line[strlen(structure)+strlen(field)+4]), 256);
+                int len = 257;
+                cpuid_info.osname = (char*) malloc(len * sizeof(char));
+                strncpy(cpuid_info.osname, value, len);
                 cpuid_info.osname[strlen(value)-1] = '\0';
             }
             else if (strcmp(field, "stepping") == 0)
@@ -492,8 +504,9 @@ readTopologyFile(const char* filename, cpu_set_t cpuSet)
             else if (strcmp(field, "features") == 0)
             {
                 strcpy(value,&(line[strlen(structure)+strlen(field)+4]));
-                cpuid_info.features = (char*) malloc((strlen(value)+1) * sizeof(char));
-                strncpy(cpuid_info.features, value, strlen(value));
+		int len = 257;
+                cpuid_info.features = (char*) malloc(len * sizeof(char));
+                strncpy(cpuid_info.features, value, len);
                 cpuid_info.features[strlen(value)-1] = '\0';
             }
         }
@@ -577,6 +590,65 @@ cpu_count(cpu_set_t* set)
     return s;
 }
 
+int
+likwid_cpu_online(int cpu_id)
+{
+    if (cpu_id < 0)
+        return 0;
+    int state = 0;
+    bstring bpath = bformat("/sys/devices/system/cpu/cpu%d/online", cpu_id);
+    FILE* fp = fopen(bdata(bpath), "r");
+    if (fp)
+    {
+        char buf[10];
+        int ret = fread(buf, sizeof(char), 9, fp);
+        fclose(fp);
+        if (ret > 0)
+        {
+            state = atoi(buf);
+        }
+    }
+    else
+    {
+        fp = fopen("/sys/devices/system/cpu/online", "r");
+        if (fp)
+        {
+            char buf[100];
+            int ret = fread(buf, sizeof(char), 99, fp);
+            fclose(fp);
+            if (ret > 0)
+            {
+                bdestroy(bpath);
+                buf[ret] = '\0';
+                bpath = bfromcstr(buf);
+                struct bstrList* first = bsplit(bpath, ',');
+                for (int i = 0; i < first->qty; i++)
+                {
+                    struct bstrList* second = bsplit(first->entry[i], '-');
+                    if (second->qty == 1)
+                    {
+                        int core = atoi(bdata(second->entry[0]));
+                        if (core == cpu_id)
+                            state = 1;
+                    }
+                    else if (second->qty == 2)
+                    {
+                        int s = atoi(bdata(second->entry[0]));
+                        int e = atoi(bdata(second->entry[1]));
+                        if (cpu_id >= s && cpu_id <= e)
+                            state = 1;
+                    }
+                    bstrListDestroy(second);
+                }
+                bstrListDestroy(first);
+
+            }
+        }
+    }
+    bdestroy(bpath);
+    return state;
+}
+
 
 int
 topology_setName(void)
@@ -793,16 +865,31 @@ topology_setName(void)
                     cpuid_info.short_name = short_goldmontplus;
                     break;
 
-                case ICELAKE:
+                case ICELAKE1:
+                case ICELAKE2:
+                    cpuid_info.supportClientmem = 1;
                     cpuid_info.name = icelake_str;
                     cpuid_info.short_name = short_icelake;
                     break;
 
+                case ICELAKEX1:
+                case ICELAKEX2:
+                    cpuid_info.name = icelakesp_str;
+                    cpuid_info.short_name = short_icelakesp;
+                    break;
+
 /*                case SNOWRIDGEX:*/
 /*                    cpuid_info.name = snowridgex_str;*/
 /*                    cpuid_info.short_name = short_snowridgex;*/
 /*                    break;*/
 
+                case TIGERLAKE1:
+                case TIGERLAKE2:
+                    //cpuid_info.supportClientmem = 1;
+                    cpuid_info.name = tigerlake_str;
+                    cpuid_info.short_name = short_tigerlake;
+                    break;
+
                 default:
                     cpuid_info.name = unknown_intel_str;
                     cpuid_info.short_name = short_unknown;
@@ -941,12 +1028,31 @@ topology_setName(void)
                     cpuid_info.name = amd_zen_str;
                     cpuid_info.short_name = short_zen;
                     break;
+                case ZENPLUS_RYZEN:
+                case ZENPLUS_RYZEN2:
+                    cpuid_info.name = amd_zenplus_str;
+                    cpuid_info.short_name = short_zen;
+                    break;
                 case ZEN2_RYZEN:
+                case ZEN2_RYZEN2:
                     cpuid_info.name = amd_zen2_str;
                     cpuid_info.short_name = short_zen2;
                     break;
             }
             break;
+        case ZEN3_FAMILY:
+            switch (cpuid_info.model)
+            {
+                case ZEN3_RYZEN:
+                    cpuid_info.name = amd_zen3_str;
+                    cpuid_info.short_name = short_zen3;
+                    break;
+                case ZEN3_RYZEN2:
+                    cpuid_info.name = amd_zen3_str;
+                    cpuid_info.short_name = short_zen3;
+                    break;
+            }
+            break;
 
         case ARMV7_FAMILY:
             switch (cpuid_info.model)
@@ -975,6 +1081,10 @@ topology_setName(void)
                             cpuid_info.name = arm_cortex_a53;
                             cpuid_info.short_name = short_arm8;
                             break;
+                        case ARM_NEOVERSE_N1:
+                            cpuid_info.name = arm_neoverse_n1;
+                            cpuid_info.short_name = short_arm8_neo_n1;
+                            break;
                         default:
                             return EXIT_FAILURE;
                             break;
@@ -1001,6 +1111,17 @@ topology_setName(void)
                             break;
                     }
                     break;
+                case FUJITSU_ARM:
+                    switch (cpuid_info.part)
+                    {
+                        case FUJITSU_A64FX:
+                            cpuid_info.name = fujitsu_a64fx;
+                            cpuid_info.short_name = short_a64fx;
+                            break;
+                        default:
+                            return EXIT_FAILURE;
+                            break;
+                    }
                 default:
                     return EXIT_FAILURE;
                     break;
@@ -1313,6 +1434,8 @@ print_supportedCPUs (void)
     printf("\t%s\n",kabylake_str);
     printf("\t%s\n",coffeelake_str);
     printf("\t%s\n",cascadelakeX_str);
+    printf("\t%s\n",tigerlake_str);
+    printf("\t%s\n",icelake_str);
     printf("\n");
     printf("Supported AMD processors:\n");
     printf("\t%s\n",opteron_sc_str);
@@ -1326,12 +1449,14 @@ print_supportedCPUs (void)
     printf("\t%s\n",kabini_str);
     printf("\t%s\n",amd_zen_str);
     printf("\t%s\n",amd_zen2_str);
+    printf("\t%s\n",amd_zen3_str);
     printf("\n");
     printf("Supported ARMv8 processors:\n");
     printf("\t%s\n",arm_cortex_a53);
     printf("\t%s\n",arm_cortex_a57);
     printf("\t%s\n",cavium_thunderx_str);
     printf("\t%s\n",cavium_thunderx2t99_str);
+    printf("\t%s\n",fujitsu_a64fx);
     printf("\n");
     printf("Supported ARMv7 processors:\n");
     printf("\t%s\n",armv7l_str);
