@@ -5,14 +5,14 @@
  *
  *      Description:  Interface to the hwloc based topology backend
  *
- *      Version:   5.0
- *      Released:  10.11.2019
+ *      Version:   5.1.0
+ *      Released:  20.11.2020
  *
  *      Authors:  Thomas Gruber (tr), thomas.roehl@googlemail.com
  *
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -180,6 +180,7 @@ int parse_cpuname(char *name)
 }
 #endif
 
+
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 #if !defined(__ARM_ARCH_7A__) && !defined(__ARM_ARCH_8A) && !defined(_ARCH_PPC)
 static int
@@ -245,7 +246,8 @@ hwloc_init_cpuInfo(cpu_set_t cpuSet)
     {
         likwid_hwloc_topology_init(&hwloc_topology);
 #if HWLOC_API_VERSION > 0x00020000
-        likwid_hwloc_topology_set_flags(hwloc_topology, HWLOC_TOPOLOGY_FLAG_WHOLE_SYSTEM );
+        likwid_hwloc_topology_set_flags(hwloc_topology, HWLOC_TOPOLOGY_FLAG_INCLUDE_DISALLOWED );
+        likwid_hwloc_topology_set_type_filter(hwloc_topology, HWLOC_OBJ_PCI_DEVICE, HWLOC_TYPE_FILTER_KEEP_ALL);
 #else
         likwid_hwloc_topology_set_flags(hwloc_topology, HWLOC_TOPOLOGY_FLAG_WHOLE_SYSTEM|HWLOC_TOPOLOGY_FLAG_WHOLE_IO );
 #endif
@@ -292,7 +294,7 @@ hwloc_init_cpuInfo(cpu_set_t cpuSet)
     }
     snprintf(cpuid_info.architecture, 19, "armv7");
 #endif
-#ifdef __ARM_ARCH_8A
+#if defined(__ARM_ARCH_7A__) || defined(__ARM_ARCH_8A)
     uint32_t part = 0;
     uint32_t count = 0;
     parse_cpuinfo(&count, &cpuid_info.family, &cpuid_info.model, &cpuid_info.stepping, &cpuid_info.part, &cpuid_info.vendor);
@@ -427,6 +429,10 @@ hwloc_init_nodeTopology(cpu_set_t cpuSet)
             continue;
         }
         id = obj->os_index;
+
+        if (id < 0 || id >= cpuid_topology.numHWThreads)
+            continue;
+
         if (CPU_ISSET(id, &cpuSet))
         {
             hwThreadPool[id].inCpuSet = 1;
@@ -435,6 +441,12 @@ hwloc_init_nodeTopology(cpu_set_t cpuSet)
         {
             hwThreadPool[id].inCpuSet = 1;
         }
+
+        if (!likwid_cpu_online(obj->os_index))
+        {
+            hwThreadPool[id].inCpuSet = 0;
+        }
+
         hwThreadPool[id].apicId = obj->os_index;
         hwThreadPool[id].threadId = obj->sibling_rank;
         if (maxNumLogicalProcsPerCore > 1)
@@ -519,7 +531,7 @@ hwloc_init_nodeTopology(cpu_set_t cpuSet)
                             hwThreadPool[id].inCpuSet)
     }
 
-    int socket_nums[16];
+    int socket_nums[MAX_NUM_NODES];
     int num_sockets = 0;
     for (uint32_t i=0; i< cpuid_topology.numHWThreads; i++)
     {
